/*

Copyright (C) 2005, 2006 John L. Weatherwax

This file is part of Octave.

Octave is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

Octave is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with Octave; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
   
Written by:
-- 
John L. Weatherwax                2006-03-05
   
email: wax@alum.mit.edu
   
Please send comments and especially bug reports to the
above email address.
   
*/

#include <octave/oct.h>
#include <octave/parse.h>
#include <octave/f77-fcn.h>

// These are the declarations of the routines that the FORTRAN code will call:
//
typedef F77_RET_T (*pc_f_ptr) (const double&, const double&, const double&, double*, double*, double*, \
                               double*, double*, double*, const int& );
typedef F77_RET_T (*pc_diff_h_ptr) (const double&, const double&, const double&, double*, double*, const int& );
typedef F77_RET_T (*pc_diff_v_ptr) (const double&, const double&, const double&, double*, double*, const int& );
typedef F77_RET_T (*pc_bndry_h_ptr) (const double&, const double&, const double&, double*, double*, double*, double*, const int& );
typedef F77_RET_T (*pc_bndry_v_ptr) (const double&, const double&, const double&, double*, double*, double*, double*, const int& );

// These are the declarations of the FORTRAN subroutine that the octave C++ gateway routine
// will call:
//
extern "C" {
  int F77_FUNC(drivep,DRIVEP) ( int&, double&, double&, double*, double&, double&, int&, \
                                int&, double*, int*, double*, double*, \
				pc_f_ptr, \
				pc_diff_h_ptr, pc_diff_v_ptr, \
                                pc_bndry_h_ptr, pc_bndry_v_ptr );
}

// Global static octave function pointers to the user defined functions
// required by pdetwo:
//
static octave_function *pdetwo_f_fcn = 0;
static octave_function *pdetwo_diff_h_fcn = 0;
static octave_function *pdetwo_diff_v_fcn = 0; 
static octave_function *pdetwo_bndry_v_fcn = 0; 
static octave_function *pdetwo_bndry_h_fcn = 0; 
static octave_function *pdetwo_uinit_fcn = 0;

// The following are groupings of pairs of routines (one after each other):
// 
// 1) C++ octave API calls of each user function:
// --These enable calls of the user functions from within the C++
//   environment ... mainly for testing
// 2) C++ octave class <-> FORTRAN conversion subroutine/function
// --These are the functional prototypes called by the FORTRAN
// 


//
// Code for "EXTERNAL F"
// 
ColumnVector
pdetwo_f_function (double t, double x, double y, const ColumnVector& u, 
                   const ColumnVector & ux, const ColumnVector & uy, 
		   const Matrix& uxx, const Matrix& uyy)
{
  ColumnVector retval;

  octave_value_list args;
  args(0) = t; args(1) = x; args(2) = y; args(3) = u; args(4) = ux; args(5) = uy; args(6) = uxx; args(7) = uyy; 

  octave_stdout << "In pdetwo_f_function" << "\n";

  if (pdetwo_f_fcn)
    {
      octave_value_list tmp = pdetwo_f_fcn->do_multi_index_op (1, args);

      if (error_state)
	{
	  gripe_user_supplied_eval ("pdetwo_f");
	  return retval;
	}

      if (tmp.length () > 0 && tmp(0).is_defined ())
	{
	  retval = ColumnVector (tmp(0).vector_value ());

	  if (error_state || retval.length () == 0)
	    gripe_user_supplied_eval ("pdetwo_f");
	}
      else
	gripe_user_supplied_eval ("pdetwo_f");
    }

  return retval;
}

// This is the subroutine that will be compiled and called by the FORTRAN driver
// routine pdetwo.f.
static F77_RET_T
pdetwo_fortran_f_func(const double& T, const double& X, const double& Y, 
		      double* U, double* UX, double* UY, double* UXX, double* UYY, 
		      double* DUDT, const int& NPDE)
{
  octave_value_list args;
  args(0) = T; args(1) = X; args(2) = Y; 

  ColumnVector Uv(NPDE),UXv(NPDE),UYv(NPDE),UXXv(NPDE),UYYv(NPDE);
  for( int i=0; i < NPDE; i++ ){
    Uv(i)=U[i]; UXv(i)=UX[i]; UYv(i)=UY[i]; UXXv(i)=UXX[i]; UYYv(i)=UYY[i];
  }
  args(3) = Uv; args(4) = UXv; args(5) = UYv; args(6) = UXXv; args(7) = UYYv;
  int nargout = 1; 

  //octave_stdout << "In pdetwo_fortran_f_func before feval" << "\n";
  octave_value_list retval = feval(pdetwo_f_fcn, args, nargout);
  if (! error_state && retval.length () == 1){
      Matrix DUDTv = retval(0).matrix_value(); 
      if( !error_state ){
	for (int i = 0; i < NPDE; i++){
	  //octave_stdout << "DUDTv(" << i << ") = " << DUDTv(i) << "\n"; 
	  DUDT[i] = DUDTv(i);
	}
      }else{
	error("failure evaluating user-supplied function F");
      }
  }
  else
    error ("failure evaluating user-supplied function F");

  F77_RETURN(0);
}


//
// Code for "EXTERNAL DIFF_H"
//
Matrix
pdetwo_diff_h_function (double t, double x, double y, const ColumnVector& u)
{
  Matrix retval;

  octave_value_list args;
  args(0) = t; args(1) = x; args(2) = y; args(3) = u;

  if (pdetwo_diff_h_fcn)
    {
      octave_value_list tmp = pdetwo_diff_h_fcn->do_multi_index_op (1, args);

      if (error_state)
	{
	  gripe_user_supplied_eval ("pdetwo_diff_h");
	  return retval;
	}

      if (tmp.length () > 0 && tmp(0).is_defined ())
	{
	  retval = Matrix (tmp(0).matrix_value ());

	  if (error_state || retval.length () == 0)
	    gripe_user_supplied_eval ("pdetwo_diff_h");
	}
      else
	gripe_user_supplied_eval ("pdetwo_diff_h");
    }

  return retval;
}

// This is the subroutine that will be compiled and called by the FORTRAN driver
// routine pdetwo.f.
static F77_RET_T
pdetwo_fortran_diff_h_func(const double& T, const double& X, const double& Y,
			   double* U, double* UH, const int& NPDE){
  octave_value_list args;

  // Prepare for a calls to pdetwo_diff_h_fcn, pdetwo_dv_fcn, and pdetwo_dzdt_fcn: 
  args(0) = T; args(1) = X; args(2) = Y; 
  ColumnVector Uv(NPDE);
  for( int i=0; i < NPDE; i++ ) Uv(i)=U[i];
  args(3) = Uv;
  int nargout = 1; 

  //octave_stdout << "In pdetwo_fortran_diff_h before feval" << "\n";
  octave_value_list retval = feval(pdetwo_diff_h_fcn, args, nargout);
  if( !error_state && retval.length() == 1 ){
    Matrix UHm = retval(0).matrix_value(); 
    if( !error_state ){
      for (int i = 0; i < NPDE*NPDE; i++){
	//octave_stdout << "UHm(" << i << ") = " << UHm(i) << "\n"; 
	UH[i] = UHm(i);
      }
    }else{
      error("failure evaluating user-supplied function DIFF_H");
    }
  }
  else
    error("failure evaluating user-supplied function DIFF_H");

  F77_RETURN(0);
}

//
// Code for "EXTERNAL DIFF_V"
//
Matrix
pdetwo_diff_v_function (double t, double x, double y, const ColumnVector& u)
{
  Matrix retval;

  octave_value_list args;
  args(0) = t; args(1) = x; args(2) = y; args(3) = u;

  if (pdetwo_diff_v_fcn)
    {
      octave_value_list tmp = pdetwo_diff_v_fcn->do_multi_index_op (1, args);

      if (error_state)
	{
	  gripe_user_supplied_eval ("pdetwo_diff_v");
	  return retval;
	}

      if (tmp.length () > 0 && tmp(0).is_defined ())
	{
	  retval = Matrix (tmp(0).matrix_value ());

	  if (error_state || retval.length () == 0)
	    gripe_user_supplied_eval ("pdetwo_diff_v");
	}
      else
	gripe_user_supplied_eval ("pdetwo_diff_v");
    }

  return retval;
}

// This is the subroutine that will be compiled and called by the FORTRAN driver
// routine pdetwo.f.
static F77_RET_T
pdetwo_fortran_diff_v_func(const double& T, const double& X, const double& Y,
			   double* U, double* UV, const int& NPDE){
  octave_value_list args;

  // Prepare for a calls to pdetwo_diff_v_fcn:
  args(0) = T; args(1) = X; args(2) = Y; 
  ColumnVector Uv(NPDE);
  for( int i=0; i < NPDE; i++ ) Uv(i)=U[i];
  args(3) = Uv;
  int nargout = 1; 

  //octave_stdout << "In pdetwo_fortran_diff_v before feval" << "\n";
  octave_value_list retval = feval(pdetwo_diff_v_fcn, args, nargout);
  if( !error_state && retval.length() == 1 ){
    Matrix UVm = retval(0).matrix_value(); 
    if( !error_state ){
      for (int i = 0; i < NPDE*NPDE; i++){
	//octave_stdout << "UVm(" << i << ") = " << UVm(i) << "\n"; 
	UV[i] = UVm(i);
      }
    }else{
      error("failure evaluating user-supplied function DIFF_V");
    }
  }
  else
    error("failure evaluating user-supplied function DIFF_V");
  
  F77_RETURN(0);
}

//
// Code for "EXTERNAL BNDRY_H"
//
Matrix
pdetwo_bndry_h_function (double t, double x, double y, const ColumnVector& u)
{
  Matrix retval;

  octave_value_list args;
  args(0) = t; args(1) = x; args(2) = y; args(3) = u;

  if (pdetwo_bndry_h_fcn)
    {
      octave_value_list tmp = pdetwo_bndry_h_fcn->do_multi_index_op (1, args);

      if (error_state)
	{
	  gripe_user_supplied_eval ("pdetwo_bndry_h");
	  return retval;
	}

      if (tmp.length () > 0 && tmp(0).is_defined ())
	{
	  retval = Matrix (tmp(0).matrix_value());
	  retval = retval.append(tmp(1).matrix_value()); 
	  retval = retval.append(tmp(2).matrix_value()); 

	  if (error_state || retval.length () == 0)
	    gripe_user_supplied_eval ("pdetwo_bndry_h");
	}
      else
	gripe_user_supplied_eval ("pdetwo_bndry_h");
    }

  return retval;
}

// This is the subroutine that will be compiled and called by the FORTRAN driver
// routine pdetwo.f.
static F77_RET_T
pdetwo_fortran_bndry_h_func(const double& T, const double& X, const double& Y,
			    double* U, double* AH, double* BH, double* CH, const int& NPDE){
  octave_value_list args;

  // Prepare for a calls to pdetwo_bndry_h_fcn, pdetwo_dv_fcn, and pdetwo_dzdt_fcn: 
  args(0) = T; args(1) = X; args(2) = Y; 
  ColumnVector Uv(NPDE);
  for( int i=0; i < NPDE; i++ ) Uv(i)=U[i];
  args(3) = Uv;
  int nargout = 1; 

  //octave_stdout << "In pdetwo_fortran_bndry_h before feval" << "\n";
  octave_value_list retval = feval(pdetwo_bndry_h_fcn, args, nargout);
  if( !error_state && retval.length() == 3 ){
    Matrix ah = retval(0).matrix_value(); 
    Matrix bh = retval(1).matrix_value(); 
    Matrix ch = retval(2).matrix_value(); 
    if( error_state ) error("failure evaluating user-supplied function BNDRY_H");
    for (int i = 0; i < NPDE; i++){
      //octave_stdout << "ah(" << i << ") = " << ah(i) << "\n"; 
      AH[i] = ah(i);
      //octave_stdout << "bh(" << i << ") = " << bh(i) << "\n"; 
      BH[i] = bh(i);
      //octave_stdout << "ch(" << i << ") = " << ch(i) << "\n"; 
      CH[i] = ch(i);
    }
  }
  else
    error("failure evaluating user-supplied function BNDRY_H");
  
  F77_RETURN(0);
}

//
// Code for "EXTERNAL BNDRY_V"
//
Matrix
pdetwo_bndry_v_function (double t, double x, double y, const ColumnVector& u)
{
  Matrix retval;

  octave_value_list args;
  args(0) = t; args(1) = x; args(2) = y; args(3) = u;

  if (pdetwo_bndry_v_fcn)
    {
      octave_value_list tmp = pdetwo_bndry_v_fcn->do_multi_index_op (1, args);

      if (error_state)
	{
	  gripe_user_supplied_eval ("pdetwo_bndry_v");
	  return retval;
	}

      if (tmp.length () > 0 && tmp(0).is_defined ())
	{
	  retval = Matrix (tmp(0).matrix_value ());
	  retval = retval.append(tmp(1).matrix_value()); 
	  retval = retval.append(tmp(2).matrix_value()); 

	  if (error_state || retval.length () == 0)
	    gripe_user_supplied_eval ("pdetwo_bndry_v");
	}
      else
	gripe_user_supplied_eval ("pdetwo_bndry_v");
    }

  return retval;
}

// This is the subroutine that will be compiled and called by the FORTRAN driver
// routine pdetwo.f.
static  F77_RET_T
pdetwo_fortran_bndry_v_func(const double& T, const double& X, const double& Y,
			    double* U, double* AV, double* BV, double* CV, const int& NPDE){
  octave_value_list args;

  // Prepare for a calls to pdetwo_bndry_v_fcn, pdetwo_dv_fcn, and pdetwo_dzdt_fcn: 
  args(0) = T; args(1) = X; args(2) = Y; 
  ColumnVector Uv(NPDE);
  for( int i=0; i < NPDE; i++ ) Uv(i)=U[i];
  args(3) = Uv;
  int nargout = 1; 

  //octave_stdout << "In pdetwo_fortran_bndry_v before feval" << "\n";
  octave_value_list retval = feval(pdetwo_bndry_v_fcn, args, nargout);
  if( !error_state && retval.length() == 3 ){
    Matrix av = retval(0).matrix_value(); 
    Matrix bv = retval(1).matrix_value(); 
    Matrix cv = retval(2).matrix_value(); 
    if( error_state ) error("failure evaluating user-supplied function BNDRY_V");
    for (int i = 0; i < NPDE; i++){
      //octave_stdout << "av(" << i << ") = " << av(i) << "\n"; 
      AV[i] = av(i);
      //octave_stdout << "bv(" << i << ") = " << bv(i) << "\n"; 
      BV[i] = bv(i);
      //octave_stdout << "cv(" << i << ") = " << cv(i) << "\n"; 
      CV[i] = cv(i);
    }
  }
  else
    error("failure evaluating user-supplied function BNDRY_V");
  
  F77_RETURN(0);
}

#ifdef _UINIT_FN_
//
// Code for "EXTERNAL UINIT"
// 
ColumnVector
pdetwo_uinit_function (double x, double y)
{
  ColumnVector retval;

  octave_value_list args;
  args(0) = x; args(1) = y; 

  if (pdetwo_uinit_fcn)
    {
      octave_value_list tmp = pdetwo_uinit_fcn->do_multi_index_op (1, args);

      if (error_state)
	{
	  gripe_user_supplied_eval ("pdetwo_uinit");
	  return retval;
	}

      if (tmp.length () > 0 && tmp(0).is_defined ())
	{
	  retval = ColumnVector (tmp(0).vector_value ());

	  if (error_state || retval.length () == 0)
	    gripe_user_supplied_eval ("pdetwo_uinit");
	}
      else
	gripe_user_supplied_eval ("pdetwo_uinit");
    }

  return retval;
}
#endif

void
Indicate_Any_Abnormal_Results(int INDEX){
  switch(INDEX){
  case 0:
    /* no error...do nothing */ 
    break; 
  case -1: 
    error("Integration halted after failing to pass the error test after reducing h by a factor of 10"); break;
  case -2: 
    error("After some initial success the integration was halted after"); break;
  case -3: 
    error("The integration was halted after failing to achieve corrector convergence"); break;
  case -4:
    error("Illegal input"); break;
  case -5:
    error("Index was -1 on input but tout was not beyond t"); break;
  case -6:
    error("Insufficient storage"); break;
  default:
    error("Unknown value of INDEX");
  }
};

DEFUN_DLD(pdetwog, args, nargout, "pdetwo: an algorithm for solving two dimensional PDE's of\
the form\n\
u_t = F(t,x,y,u,u_x,u_y,u_xx,u_yy)"){

  octave_value_list retval; 

  // Input argument checking: 
  int nargin=args.length(); 
  if( nargin != 13 ){
    error("pdetwog: Incorrect number of input arguments");
    print_usage("pdetwog"); 
    return retval;
  }
  if( nargout < 1 ){ 
    error( "pdetwog: Incorrect number of output arguments");
    print_usage("pdetwog"); 
    return retval;
  }

  //
  // Extract all inputs:
  //
  RowVector touts( args(0).vector_value() );
  double t0 = touts(0); // the initial time
  double *toutsP = touts.fortran_vec();
  int ntouts = args(0).length();
  //octave_stdout << "ntouts = " << ntouts << "\n"; 
  double H = args(1).double_value();
  RowVector xbkpt( args(2).vector_value() );
  double *XBKPTP = xbkpt.fortran_vec();
  int NX = args(2).length();
  //octave_stdout << "NX = " << NX << "\n"; 
  RowVector ybkpt( args(3).vector_value() );
  double *YBKPTP = ybkpt.fortran_vec();
  int NY = args(3).length();
  //octave_stdout << "NY = " << NY << "\n"; 
  double EPS = args(4).double_value();
  int MF = args(5).int_value();
  int MORDER = args(6).int_value();

  // testing the user supplied functions...debugging...
  octave_value f_arg;
  //octave_stdout << "Using hardwired value of NPDE=1\n"; 
  int NPDE;  

//#define _TESTING_ 

  // Extract a octave function pointer to the "F" function:
  // 
  f_arg = args(7);
  pdetwo_f_fcn = extract_function
	    (f_arg, "pdetwo", "__pdetwo_f_fcn__",
	     "function F = __pdetwo_f_fcn__ (t, x, y, u, ux, uy, duxx, duyy) F = ",
	     "; endfunction");
#ifdef _TESTING_ 
  ColumnVector testV(NPDE,-3.14159),dumU(NPDE,2.7183);
  Matrix testM(NPDE,NPDE,+3.14159),dumDUXX(NPDE,NPDE,2.7183);
  testV = pdetwo_f_function(1.0,-1.0,1.0,dumU,dumU,dumU,dumDUXX,dumDUXX);
  octave_stdout << "pdetwo_f_function = \n" << testV << "\n"; 
#endif

  // Extract a octave function pointer to the "DIFF_H" function:
  // 
  f_arg = args(8);
  pdetwo_diff_h_fcn = extract_function
         (f_arg, "pdetwo", "__pdetwo_diff_h_fcn__",
          "function DIFF_H = __pdetwo_diff_h_fcn__ (t, x, y, u) DIFF_H = ",
          "; endfunction"); 
#ifdef _TESTING_
  // See if I can call the user function "DIFF_H":
  testM = pdetwo_diff_h_function(1.0,-1.0,+1.0,dumU);
  octave_stdout << "pdetwo_diff_h_function = \n" << testM << "\n"; 
#endif

  // Extract a octave function pointer to the "DIFF_V" function:
  // 
  f_arg = args(9);
  pdetwo_diff_v_fcn = extract_function
         (f_arg, "pdetwo", "__pdetwo_diff_v_fcn__",
          "function DIFF_V = __pdetwo_diff_v_fcn__ (t, x, y, u) DIFF_V = ",
          "; endfunction"); 
#ifdef _TESTING_
  // See if I can call the user function "DIFF_V":
  testM = pdetwo_diff_v_function(1.0,-1.0,+1.0,dumU);
  octave_stdout << "pdetwo_diff_v_function = \n" << testM << "\n"; 
#endif

  // Extract a octave function pointer to the "BNDRY_H" function:
  // 
  f_arg = args(10);
  pdetwo_bndry_h_fcn = extract_function
         (f_arg, "pdetwo", "__pdetwo_bndry_h_fcn__",
          "function BNDRY_H = __pdetwo_bndry_h_fcn__ (t, x, y, u) BNDRY_H = ",
          "; endfunction"); 
#ifdef _TESTING_
  // See if I can call the user function "BNDRY_H":
  testM = pdetwo_bndry_h_function(1.0,-1.0,+1.0,dumU);
  octave_stdout << "pdetwo_bndry_h_function = \n" << testM << "\n"; 
#endif

  // Extract a octave function pointer to the "BNDRY_V" function:
  // 
  f_arg = args(11);
  pdetwo_bndry_v_fcn = extract_function
         (f_arg, "pdetwo", "__pdetwo_bndry_v_fcn__",
          "function BNDRY_V = __pdetwo_bndry_v_fcn__ (t, x, y, u) BNDRY_H = ",
          "; endfunction"); 
#ifdef _TESTING_
  // See if I can call the user function "BNDRY_V":
  testM = pdetwo_bndry_v_function(1.0,-1.0,+1.0,dumU);
  octave_stdout << "pdetwo_bndry_v_function = \n" << testM << "\n"; 
#endif

f_arg = args(12);
#ifdef _UINIT_FN_
  // Extract a octave function pointer to the "UINIT" function:
  // 
  pdetwo_uinit_fcn = extract_function
         (f_arg, "pdetwo", "__pdetwo_uinit_fcn__",
          "function UINIT = __pdetwo_uinit_fcn__ (x,y) UINIT = ",
          "; endfunction"); 
  // Call the function "UINIT" to determine the number of equations we are integrating:
  // Assume that the midpoint of the xbkpt vector is an allowable evaluation point:
  ColumnVector uinit_out = pdetwo_uinit_function( 0.5*(xbkpt(0) + xbkpt(xbkpt.length()-1)),
                                                  0.5*(ybkpt(0) + ybkpt(ybkpt.length()-1)) );
  //int NPDE = uinit_out.length(); 
#ifdef _TESTING_
  octave_stdout << "uinit_out = \n" << uinit_out << "\n"; 
#endif
#else
  // Extract initial conditions NDArray from input:
  // 
  NDArray U0( f_arg.array_value() );
  dim_vector dvin = U0.dims(); 
  //octave_stdout << "U0 = " << U0 << "\n";
  //octave_stdout << "length( dvin ) = " << dvin.length() << "\n"; 
  //octave_stdout << "dvin(0) = " << dvin(0) << "\n"; 
  //octave_stdout << "dvin(1) = " << dvin(1) << "\n"; 
  int NX0,NY0; 
  if( dvin.length() == 2 ){ // A of size [nRows,nCols] has (nx,ny)=(nCols,nRows)
    NPDE=1;       NX0=dvin(1); NY0=dvin(0); 
  } else {
    NPDE=dvin(0); NX0=dvin(2); NY0=dvin(1); 
  }
  if( ( NX0 != NX ) || ( NY0 != NY ) ){
    octave_stdout << "NX0=" << NX0 << "; NY0=" << NY0 << "\n";
    octave_stdout << "NX=" << NX << "; NY=" << NY << "\n";
    error( "pdetwog: the dimension of the initial conditions does not match that of the grid" );
    return retval; 
  }
#endif

  // Calculate the required auxiliary data arrays sizes:
  //
  int NODE  = NX*NY*NPDE;
  int METH  = MF/10;
  int MITER = MF - 10*METH;

  int LPW; 
  switch(MITER){
    case 0: LPW=1; break; 
    case 1: LPW=(3*(NX+1)*NPDE-2)*NODE; break; 
    case 2: LPW=(3*(NX+1)*NPDE-2)*NODE; break; 
    case 3: LPW=NODE*(MORDER+1); break; 
    default:
      error( "Incorrect value for MITER.  Choices are 0, 1, 2, 3." ); 
      return retval; 
  }
  int LU = NODE*(MORDER+1);
  int NRWK  = NPDE * ( NPDE*(NX*2+3) + 13 + NX ) + NX*4 + 4*NODE + LPW + LU;
  int NRIWK = NODE;

  // Allocate storage for work/iwork to pass into FORTRAN code:
  //
  OCTAVE_LOCAL_BUFFER( double, WORKP, NRWK ); 
  OCTAVE_LOCAL_BUFFER( int, IWORKP, NRIWK );
  IWORKP[0] = NPDE;
  IWORKP[1] = NX;
  IWORKP[2] = NY;
  IWORKP[3] = MORDER;
  IWORKP[4] = NRWK;
  IWORKP[5] = NRIWK;  

  // Check user inputs for correctness: 
  //
  if( !(MF==10 || MF==11 || MF==12 || MF==13 || MF==20 || MF==21 || MF==22 || MF==23) ){
    error( "Incorrect value for MF.  Choices are 10, 11, 12, 13, 20, 21, 22, 23." ); 
    return retval; 
  }
  if( METH==1 && MORDER>12 ){
    error( "The maximum order of the method used in the modified GEARB must be less than or equal to 12 if METH=1" );
    return retval; 
  }
  if( METH==2 && MORDER>12 ){
    error( "The maximum order of the method used in the modified GEARB must be less than or equal to 5 if METH=2" );
    return retval; 
  }

  // u T(otal) (i.e. all timesteps) is declared as an "NDArray":
  dim_vector dvp(NPDE,NX,NY); // dimension of each solution page
  dim_vector dvt(NPDE,NX,NY); // will become the dimension of the total solution
  //octave_stdout << "ntouts=" << ntouts << "\n"; 

  // Concatenate the appropriate number of pages to store the entire solution (for all timesteps)
  // 
  for ( int tcnt = 0; tcnt < ntouts-1; tcnt++ ){ 
    dvt.concat( dvp, 3 ); 
  }
  //octave_stdout << "dvt.length() = " << dvt.length() << "\n";
  //octave_stdout << "dvt(0)=" << dvt(0) << ";dvt(1)=" << dvt(1) << ";dvt(2)=" << dvt(2) << ";dvt(3)=" << dvt(3) << "\n";

  NDArray uT( dvt );
  Array<int> dim(4,0);
  // u T(ime) S(tep) solution: 
  NDArray uTS( dvp );

  // Fill uTS with the initial conditions matrix:
  // (there maybe easier ways to do this but this works so ...):
  double *utsp = uTS.fortran_vec(); 
  for( int i=0; i < U0.nelem(); i++ ){
    utsp[i]=U0(i);
  }
  //octave_stdout << "-----\n"; 
  //octave_stdout << "uTS = " << uTS << "\n"; 

  dim(3)=0;
  uT.concat(uTS,dim); 
  //octave_stdout << "-----\n"; 
  //octave_stdout << "uT = " << uT << "\n"; 

  // Loop, calling DRIVEP repeatedly:
  // 
  int INDEX=1;
  for( int tcnt = 0; tcnt < ntouts-1; tcnt++ ){
    double T0=touts(tcnt);
    double TOUT=touts(tcnt+1);
    //octave_stdout << "Before drivep call..." << "\n";
    //octave_stdout << "T0=" << T0 << "; TOUT=" << TOUT << "\n";
    F77_FUNC(drivep,DRIVEP)(NODE,T0,H,uTS.fortran_vec(),TOUT,EPS,MF,                   \
 			    INDEX,WORKP,IWORKP,XBKPTP,YBKPTP,	                       \
 			    pdetwo_fortran_f_func,                                     \
			    pdetwo_fortran_diff_h_func,pdetwo_fortran_diff_v_func,     \
 			    pdetwo_fortran_bndry_h_func,pdetwo_fortran_bndry_v_func);
    Indicate_Any_Abnormal_Results(INDEX);
    //octave_stdout << "Finished cycle ...\n";
    //octave_stdout << "uTS = " << uTS << "\n";

    // Assign the location where this page of results goes: 
    dim(3)=tcnt+1; 
    //octave_stdout << "dim(0)=" << dim(0) << ";dim(1)=" << dim(1) << ";dim(2)=" << dim(2) << ";dim(3)=" << dim(3) << "\n"; 
    uT.concat(uTS,dim); 
  }
  // Store to pass out:
  retval.append(uT);
  return retval;
}
